@@ -1,5 +1,6 @@
 from dexy.constants import Constants
 import StringIO
+import codecs
 import dexy.controller
 import dexy.introspect
 import fnmatch
@@ -7,6 +8,7 @@ import hashlib
 import json
 import logging
 import os
+import re
 import shutil
 import time
 import urllib2
@@ -125,10 +127,12 @@ class Document(object):
                         # The relative path doc key is specified
                         is_exact_relative_match = (os.path.join(relpath2, input_glob) == doc.key())
 
+                        is_exact_match_in_parent_or_child_dir = (in_parent_or_child and input_glob == os.path.basename(doc.key()))
+
                         # A glob matches in any child dir
                         is_glob_match_in_child_dir = (in_parent_dir or in_same_dir) and fnmatch.fnmatch(os.path.basename(doc.key()), input_glob)
 
-                        specified = (is_exact_absolute_match or is_exact_relative_match or is_glob_match_in_child_dir)
+                        specified = (is_exact_absolute_match or is_exact_relative_match or is_exact_match_in_parent_or_child_dir or is_glob_match_in_child_dir)
 
                         if specified:
                             break
@@ -183,7 +187,10 @@ class Document(object):
         # potentially dangerous methods ...
         elif self.args.has_key('url'):
             url = self.args['url']
-            filename = os.path.join(self.artifacts_dir, self.name)
+
+            local_filename = re.sub('[^\w\s-]', '', url).strip().lower()
+            local_filename = re.sub('[-\s]+', '-', local_filename)
+            filename = os.path.join(self.artifacts_dir, local_filename)
             header_filename = "%s.headers" % filename
 
             if not os.path.exists(os.path.dirname(filename)):
@@ -191,7 +198,7 @@ class Document(object):
 
             header_dict = {}
             if os.path.exists(header_filename):
-                header_file = open(header_filename, "r")
+                header_file = codecs.open(header_filename, "r", encoding="utf-8")
                 header_dict = json.load(header_file)
                 header_file.close()
 
@@ -211,12 +218,12 @@ class Document(object):
             elif header_dict.has_key('Last-Modified') and os.path.exists(filename):
                 request.add_header('If-Modifed-Since', header_dict['Last-Modified'])
 
-            if self.controller.args['locals'] and os.path.exists(filename):
-                f = open(filename, "r")
+            if self.controller.args['uselocals'] and os.path.exists(filename):
+                f = codecs.open(filename, "r", encoding="utf-8")
                 data = f.read()
                 f.close()
             else:
-                if self.controller.args['locals']:
+                if self.controller.args['uselocals']:
                     print "local file %s not found, fetching remote url" % filename
 
                 try:
@@ -234,30 +241,28 @@ class Document(object):
                     for s in u.info().headers:
                         a = s.partition(":")
                         header_dict[a[0]] = a[2].strip()
-                    json.dump(header_dict, open(header_filename, "w"))
+                    json.dump(header_dict, codecs.open(header_filename, "w", encoding="utf-8"))
 
                     data = url_contents
-                except urllib2.URLError as err:
-                    if os.path.exists(filename):
-                        print "unable to fetch remote url %s because %s\nusing contents of %s" % (url, err, filename)
-                    else:
-                        raise Exception("unable to fetch remote url %s because %s\nno cache found in %s" % (url, err, filename))
+                except (urllib2.HTTPError, urllib2.URLError) as err:
+                    if not hasattr(err, 'code'):
+                        print "Trying to fetch %s for %s" % (url, self.name)
+                        raise err
+
+                    elif err.code == 304:
+                        if os.path.exists(filename):
+                            print "received NOT MODIFIED (304) from server, so using contents of %s" % (filename)
+                            f = codecs.open(filename, "r", encoding="utf-8")
+                            data = f.read()
+                            f.close()
+                        else:
+                            raise err
 
-                    f = open(filename, "r")
-                    data = f.read()
-                    f.close()
-                except urllib2.HTTPError as err:
-                    if err.code == 304:
-                        print "received http status code %s, using contents of %s" % (err.code, filename)
-                        f = open(filename, "r")
-                        data = f.read()
-                        f.close()
                     elif err.code == 404:
-                        raise Exception("""received http status code %s while trying to fetch %s for %s""" % 
-                                        (err.code, url, self.name))
+                        raise Exception("url %s NOT FOUND (404) (for virtual file %s)" % (url, self.name))
+
                     else:
-                        # Some other http error, we want to know about it.
-                        print url
+                        print "Trying to fetch %s for %s" % (url, self.name)
                         raise err
 
         elif self.args.has_key('repo') and self.args.has_key('path'):
