@@ -8,9 +8,11 @@ import re
 import uuid
 
 class TestFilter(DexyFilter):
-    """The test filter raises an error if output is not as expected. Handy for
-    testing your custom filters or for ensuring that examples in your
-    documentation stay correct."""
+    """
+    Lets you test the output of running a filter, raises exception if expectation not met.
+
+    If the test passes, the original content is returned so you can chain this filter.
+    """
 
     ALIASES = ['test']
 
@@ -21,37 +23,45 @@ class TestFilter(DexyFilter):
             return False
 
         print "testing", self.artifact.key, "...",
-        if not self.artifact.args.has_key('expects'):
-            raise "You need to pass 'expects' to the test filter."
 
-        expects = self.artifact.args['expects']
-        if expects.startswith("@"):
-            expects_file = open(expects.lstrip("@"), "r")
-            expects_content = expects_file.read()
-            expects_file.close()
-        else:
-            expects_content = expects
-        expects = expects_content
+        if self.artifact.args.has_key('test-expects'):
+            # 'expects' should be an exact match
+            expects = self.artifact.args['test-expects']
+            if expects.startswith("@"):
+                expects_file = open(expects.lstrip("@"), "r")
+                expects_content = expects_file.read()
+                expects_file.close()
+            else:
+                expects_content = expects
+            expects = expects_content
+
+            if hasattr(expects, 'items'):
+                for k, v in self.artifact.input_data_dict.items():
+                    msg = print_string_diff(expects[k], v)
+                    msg += "\nexpected output '%s' (section %s) does not match actual '%s'" % (expects[k], k, v)
+                    assert expects[k] == v, msg
+            else:
+                inp = self.artifact.input_text()
+                msg = print_string_diff(expects, inp)
+                msg += "\nexpected output '%s' does not match actual '%s'" % (expects, inp)
+                assert expects == inp, msg
+        elif self.artifact.args.has_key('test-includes'):
+            # specified text should appear somewhere in the output
+            includes_text = self.artifact.args['test-includes']
+            assert includes_text in self.artifact.input_text()
+
+        elif self.artifact.args.has_key('expects'):
+            raise Exception("'expects' is deprecated, please use 'test-expects' instead.")
 
-        # TODO handle different types of expectation, e.g. when don't know exact
-        # output but can look for type of data returned
-        if hasattr(expects, 'items'):
-            for k, v in self.artifact.input_data_dict.items():
-                msg = print_string_diff(expects[k], v)
-                msg += "\nexpected output '%s' (section %s) does not match actual '%s'" % (expects[k], k, v)
-                assert expects[k] == v, msg
         else:
-            inp = self.artifact.input_text()
-            msg = print_string_diff(expects, inp)
-            msg += "\nexpected output '%s' does not match actual '%s'" % (expects, inp)
-            assert expects == inp, msg
+            raise Exception("Must pass one of 'test-expects', 'test-includes' to test filter so there is something to test.")
 
         comment = ""
         if self.artifact.args.has_key("comment"):
            comment = "[%s]" % self.artifact.args['comment']
         print "ok", comment
 
-        # Don't change the output so we can use end result still...
+        # Return the original output, so 'test' filter doesn't change anything.
         self.artifact.data_dict = self.artifact.input_data_dict
 
 class JoinFilter(DexyFilter):
@@ -215,8 +225,12 @@ class WordWrapFilter(DexyFilter):
         return wrap_text(input_text, 79)
 
 class SplitHtmlFilter(DexyFilter):
-    """Splits a HTML page into multiple HTML pages. The original page becomes an
-    index page."""
+    """
+    Create multiple HTML pages from a single template, with an automatic index page.
+
+    The split filter looks for specially formatted HTML comments in your
+    document and splits your HTML into separate pages at each split comment.
+    """
     ALIASES = ['split', 'splithtml']
     INPUT_EXTENSIONS = [".html"]
     OUTPUT_EXTENSIONS = [".html"]
